#!/usr/bin/env python
# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Data fetchers module """

from fmriprep.data.getters import (get_mni_template,
	                               get_mni_template_ras,
	                               get_ants_oasis_template,
                                   get_ants_oasis_template_ras)
